@@ -1,5 +1,5 @@
 from typing import (
-    Any, Callable, Dict, List, Optional, Type, Union, TYPE_CHECKING
+    Any, Callable, Dict, List, Optional, Tuple, Type, Union, TYPE_CHECKING
 )
 
 import torch
@@ -27,7 +27,8 @@ def create_trainer(
         out_dir: str = 'result',
         stop_trigger: 'TriggerLike' = None,
         writer: Optional['writing.Writer'] = None,
-        evaluator: Optional['Evaluator'] = None,
+        evaluator: Optional[
+            Union['Evaluator', Tuple['Evaluator', 'TriggerLike']]] = None,
         device: 'DeviceLike' = 'cpu',
         logic: Optional[handler_module.Logic] = None,
         transform_model: Callable[
@@ -83,8 +84,8 @@ def create_trainer(
 
     options = options.copy() if options else {}
     # TODO(kmaehashi): deprecate specifying 'runtime' key in options
-    runtime_options = (
-        runtime_options.copy() if runtime_options
+    runtime_options = dict(
+        runtime_options if runtime_options
         else options.pop('runtime', {}))
     logic = handler_module.Logic() if logic is None else logic
     handler_class = handler_class if handler_class else handler_module.Handler
@@ -154,8 +155,8 @@ def create_evaluator(
     metrics = metrics if metrics else []
     options = options.copy() if options else {}
     # TODO(kmaehashi): deprecate specifying 'runtime' key in options
-    runtime_options = (
-        runtime_options.copy() if runtime_options
+    runtime_options = dict(
+        runtime_options if runtime_options
         else options.pop('runtime', {}))
     logic = handler_module.Logic() if logic is None else logic
     handler_class = handler_class if handler_class else handler_module.Handler
