@@ -1,14 +1,35 @@
-# mypy: ignore-errors
+import contextlib
 
-from typing import (
-    Any, Dict, Generator, Iterable, Optional, Tuple, Union
-)
+from typing import Any, Dict, Generator, Iterable, Optional, Tuple, Union
 
 import torch
 
+from pytorch_pfn_extras.handler._code_block import CodeBlock
 from pytorch_pfn_extras.training import Evaluator, Trainer
 
-_RUNTIME_TAG_NAME = '_ppe_runtime'
+_amp_enabled = False
+
+
+try:
+    import torch.cuda.amp
+
+    _amp_enabled = torch.cuda.is_available() and hasattr(
+        torch.cuda.amp, "autocast"
+    )
+except ImportError:
+    pass
+
+
+@contextlib.contextmanager
+def _autocast(enabled: bool = True) -> Generator[None, None, None]:
+    if _amp_enabled:
+        with torch.cuda.amp.autocast(enabled):  # type: ignore[no-untyped-call]
+            yield
+    else:
+        yield
+
+
+_RUNTIME_TAG_NAME = "_ppe_runtime"
 
 DeviceLike = Union[str, torch.device]
 
@@ -30,9 +51,7 @@ class BaseRuntime:
     """
 
     def __init__(
-            self,
-            device_spec: DeviceLike,
-            options: Optional[Dict[str, Any]] = None,
+        self, device_spec: DeviceLike, options: Dict[str, Any],
     ) -> None:
         self.device_spec = device_spec
         self.options = options
@@ -50,14 +69,12 @@ class BaseRuntime:
 
         # this should be called with the runtime associated to a model
         # or a model part
-        if isinstance(args, tuple) and hasattr(args, '_fields'):
-            return args.__class__(
-                **{k: self.move_tensor(getattr(args, k)) for k in args._fields})
+        if isinstance(args, tuple) and hasattr(args, "_fields"):
+            # namedtuple
+            return args._replace(  # type: ignore[attr-defined]
+                **self._convert_batch_dict(args._asdict()))  # type: ignore
         if isinstance(args, dict):
-            return {
-                k: self.move_tensor(v) if isinstance(v, torch.Tensor) else v
-                for k, v in args.items()
-            }
+            return self._convert_batch_dict(args)
         if isinstance(args, (list, tuple)):
             return [
                 self.move_tensor(v) if isinstance(v, torch.Tensor) else v
@@ -67,6 +84,12 @@ class BaseRuntime:
             return self.move_tensor(args)
         return args
 
+    def _convert_batch_dict(self, args: Dict[str, Any]) -> Dict[str, Any]:
+        return {
+            k: self.move_tensor(v) if isinstance(v, torch.Tensor) else v
+            for k, v in args.items()
+        }
+
     def move_module(self, module: torch.nn.Module) -> torch.nn.Module:
         """Transfers the module to the specific device.
 
@@ -93,10 +116,10 @@ class BaseRuntime:
         raise NotImplementedError()
 
     def initialize_module(
-            self,
-            module: torch.nn.Module,
-            loader_or_batch: Optional[Union[Iterable[Any], torch.Tensor]],
-            optimizer: Optional[torch.optim.Optimizer] = None,
+        self,
+        module: torch.nn.Module,
+        loader_or_batch: Optional[Union[Iterable[Any], torch.Tensor]],
+        optimizer: Optional[torch.optim.Optimizer] = None,
     ) -> None:
         """Initializes the module at the beginning of training or inference.
 
@@ -124,11 +147,11 @@ class BaseRuntime:
         raise NotImplementedError()
 
     def train_pre_step(
-            self,
-            trainer: Trainer,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
+        self,
+        trainer: Trainer,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
     ) -> None:
         """Preprocess of each step.
 
@@ -147,12 +170,12 @@ class BaseRuntime:
         raise NotImplementedError()
 
     def train_post_step(
-            self,
-            trainer: Trainer,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
-            outs: Any,
+        self,
+        trainer: Trainer,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
+        outs: Any,
     ) -> None:
         """Postprocess of each step.
 
@@ -193,11 +216,11 @@ class BaseRuntime:
         raise NotImplementedError()
 
     def eval_pre_step(
-            self,
-            evaluator: Evaluator,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
+        self,
+        evaluator: Evaluator,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
     ) -> None:
         """The method called at the beginning of each evaluation.
 
@@ -213,12 +236,12 @@ class BaseRuntime:
         raise NotImplementedError()
 
     def eval_post_step(
-            self,
-            evaluator: Evaluator,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
-            outs: Any,
+        self,
+        evaluator: Evaluator,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
+        outs: Any,
     ) -> None:
         """The method called at the end of each evaluation.
 
@@ -235,6 +258,19 @@ class BaseRuntime:
         """
         raise NotImplementedError()
 
+    def execute(self, code_block: CodeBlock, batch: Any,) -> Any:
+        """Method called by the CodeBlocks API to do device dependent execution.
+
+        Args:
+            code_block (CodeBlock): The codeblock requesting execution.
+            batch (dict of str, torch.Tensor):
+                The input tensors of this batch.
+
+        Returns:
+            The results of executing the codeblock on this runtime.
+        """
+        raise NotImplementedError()
+
 
 class PyTorchRuntime(BaseRuntime):
     """A collections of callback functions for the devices that PyTorch
@@ -242,8 +278,35 @@ class PyTorchRuntime(BaseRuntime):
 
     Args:
         device_spec (torch.device or str): The device.
+        options (dict, optional): The configuration options.
+
+            * ``'autocast'`` (bool):
+                If ``True``, ``torch.cuda.amp.autocast`` is enabled.
+                Default is ``False``.
+            * ``'grad_scaler'`` (torch.cuda.amp.GradScaler):
+                A gradient scaler that outputs are applied to.
     """
 
+    def __init__(
+        self, device_spec: DeviceLike, options: Dict[str, Any],
+    ) -> None:
+        super().__init__(device_spec, options)
+        self._grad_scaler = options.get("grad_scaler", None)
+        self._autocast = options.get("autocast", False)
+        if not _amp_enabled:
+            if self._grad_scaler is not None or self._autocast:
+                raise RuntimeError(
+                    "Requested AMP features but torch.cuda.amp"
+                    " is not enabled"
+                )
+
+        if self._grad_scaler is not None:
+            if not isinstance(self._grad_scaler, torch.cuda.amp.GradScaler):
+                raise RuntimeError(
+                    "grad_scaler should be a "
+                    "torch.cuda.amp.GradScaler object"
+                )
+
     def move_module(self, module: torch.nn.Module) -> torch.nn.Module:
         return module.to(self.device_spec)
 
@@ -251,10 +314,10 @@ class PyTorchRuntime(BaseRuntime):
         return tensor.to(self.device_spec)
 
     def initialize_module(
-            self,
-            module: torch.nn.Module,
-            loader_or_batch: Optional[Union[Iterable[Any], torch.Tensor]],
-            optimizer: Optional[torch.optim.Optimizer] = None,
+        self,
+        module: torch.nn.Module,
+        loader_or_batch: Optional[Union[Iterable[Any], torch.Tensor]],
+        optimizer: Optional[torch.optim.Optimizer] = None,
     ) -> None:
         pass
 
@@ -268,59 +331,108 @@ class PyTorchRuntime(BaseRuntime):
         pass
 
     def train_pre_step(
-            self,
-            trainer: Trainer,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
+        self,
+        trainer: Trainer,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
     ) -> None:
         pass
 
     def train_post_step(
-            self,
-            trainer: Trainer,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
-            outs: Any,
+        self,
+        trainer: Trainer,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
+        outs: Any,
     ) -> None:
         pass
 
     def eval_pre_step(
-            self,
-            evaluator: Evaluator,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
+        self,
+        evaluator: Evaluator,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
     ) -> None:
         pass
 
     def eval_post_step(
-            self,
-            evaluator: Evaluator,
-            module: torch.nn.Module,
-            batch_idx: int,
-            batch: Any,
-            outs: Any,
+        self,
+        evaluator: Evaluator,
+        module: torch.nn.Module,
+        batch_idx: int,
+        batch: Any,
+        outs: Any,
     ) -> None:
         pass
 
+    def execute(self, code_block: CodeBlock, batch: Any,) -> Any:
+        # Run forward, backward and optimize steps depending on codeblock opts
+        if self._grad_scaler is None:
+
+            def _scale(x: torch.Tensor) -> torch.Tensor:
+                return x
+
+        else:
+
+            def _scale(x: torch.Tensor) -> torch.Tensor:
+                return self._grad_scaler.scale(x)  # type: ignore[no-any-return]
+
+        if code_block.optimizer is not None:
+            code_block.optimizer.zero_grad()
+
+        # with autocast
+        with _autocast(enabled=self._autocast):
+            out = code_block.func(**batch)
+
+        # codeblocks return Dicts-per-se so it is not necessary to normalize
+        if code_block.backprop:
+            if code_block.backprop_from is None:
+                for v in out.values():
+                    if (
+                        isinstance(v, torch.Tensor)
+                        and v.grad_fn is not None
+                        and v.numel() == 1
+                        and (
+                            v.dtype.is_floating_point
+                            or v.dtype.is_complex
+                        )
+                    ):
+                        _scale(v).backward()  # type: ignore[no-untyped-call]
+            else:
+                _scale(out[code_block.backprop_from]).backward()  # type: ignore
+
+        if code_block.optimizer is None:
+            return out
+
+        if self._grad_scaler is not None:
+            self._grad_scaler.step(code_block.optimizer)
+            self._grad_scaler.update()
+        else:
+            code_block.optimizer.step()
+
+        return out
+
 
 def _module_runtime_tag(module: torch.nn.Module) -> BaseRuntime:
     return getattr(  # type: ignore[no-any-return]
-        module, _RUNTIME_TAG_NAME, None)
+        module, _RUNTIME_TAG_NAME, None
+    )
 
 
 def _set_module_runtime_tag(
-        module: torch.nn.Module, runtime: BaseRuntime) -> None:
+    module: torch.nn.Module, runtime: BaseRuntime
+) -> None:
     return setattr(module, _RUNTIME_TAG_NAME, runtime)
 
 
 def named_runtime_modules(
-        module: torch.nn.Module,
-        module_name: str = '',
-        first_level: bool = True,
-        recursive: bool = True,
+    module: torch.nn.Module,
+    module_name: str = "",
+    first_level: bool = True,
+    recursive: bool = True,
 ) -> Generator[Tuple[str, torch.nn.Module], Tuple[str, torch.nn.Module], None]:
     # This can be invoked with no containarized modules
     # to look for submodules that hold containers
