@@ -5,6 +5,9 @@ from pytorch_pfn_extras.training.extension import PRIORITY_EDITOR  # NOQA
 from pytorch_pfn_extras.training.extension import PRIORITY_READER  # NOQA
 from pytorch_pfn_extras.training.extension import PRIORITY_WRITER  # NOQA
 from pytorch_pfn_extras.training import extensions  # NOQA
+from pytorch_pfn_extras.training._manager_protocol import ExtensionsManagerProtocol  # NOQA
 from pytorch_pfn_extras.training.manager import ExtensionsManager  # NOQA
 from pytorch_pfn_extras.training.manager import IgniteExtensionsManager  # NOQA
 from pytorch_pfn_extras.training.metrics import AccuracyMetric  # NOQA
+from pytorch_pfn_extras.training._trainer import Trainer  # NOQA
+from pytorch_pfn_extras.training._evaluator import Evaluator  # NOQA
