@@ -4,6 +4,7 @@ from typing_extensions import Protocol
 import torch
 
 if TYPE_CHECKING:
+    from pytorch_pfn_extras.training import trigger as trigger_module
     from pytorch_pfn_extras.training.extension import Extension
     from pytorch_pfn_extras import writing
     from pytorch_pfn_extras import reporting
@@ -23,6 +24,10 @@ class ExtensionsManagerProtocol(Protocol):
     def epoch_detail(self) -> float:
         ...
 
+    @property
+    def _iters_per_epoch(self) -> int:
+        ...
+
     @property
     def models(self) -> Dict[str, torch.nn.Module]:
         ...
@@ -47,6 +52,10 @@ class ExtensionsManagerProtocol(Protocol):
     def stop_trigger(self) -> bool:
         ...
 
+    @property
+    def _stop_trigger(self) -> 'trigger_module.Trigger':
+        ...
+
     @property
     def out(self) -> str:
         ...
@@ -61,3 +70,7 @@ class ExtensionsManagerProtocol(Protocol):
 
     def get_extension(self, name: str) -> 'Extension':
         ...
+
+    @property
+    def observation(self) -> 'reporting.Observation':
+        ...
