@@ -1,18 +1,20 @@
-# mypy: ignore-errors
-
 import os
+import types
+from typing import Any, Generator, List, Optional, Tuple
 
 import torch
 import torch.distributed
 
 from pytorch_pfn_extras import logging
 from pytorch_pfn_extras.training import extension
+from pytorch_pfn_extras.training._manager_protocol import ExtensionsManagerProtocol
+from pytorch_pfn_extras import writing
 
 
 logger = logging._get_root_logger()
 
 
-def _find_snapshot_files(fmt, path, fs):
+def _find_snapshot_files(fmt: str, path: str, fs: Any) -> List[Tuple[float, str]]:
     '''Only prefix and suffix match
 
     TODO(kuenishi): currently clean format string such as
@@ -38,13 +40,13 @@ def _find_snapshot_files(fmt, path, fs):
     matched_files = (file for file in fs.list(path)
                      if file.startswith(prefix) and file.endswith(suffix))
 
-    def _prepend_mtime(f):
+    def _prepend_mtime(f: str) -> Any:
         t = fs.stat(os.path.join(path, f)).last_modified
         return (t, f)
     return sorted(_prepend_mtime(file) for file in matched_files)
 
 
-def _find_latest_snapshot(fmt, path, fs):
+def _find_latest_snapshot(fmt: str, path: str, fs: Any) -> Optional[str]:
     """Finds the latest snapshots in a directory
 
     Args:
@@ -68,7 +70,9 @@ def _find_latest_snapshot(fmt, path, fs):
     return None
 
 
-def _find_stale_snapshots(fmt, path, n_retains, fs):
+def _find_stale_snapshots(
+        fmt: str, path: str, n_retains: int, fs: Any
+) -> Generator[str, None, None]:
     """Finds stale snapshots in a directory, retaining several files
 
     Args:
@@ -95,7 +99,8 @@ def _find_stale_snapshots(fmt, path, n_retains, fs):
     return
 
 
-def snapshot_object(target, filename, savefun=None, **kwargs):
+def snapshot_object(
+        target: Any, filename: str, savefun: Any = None, **kwargs: Any) -> '_Snapshot':
     """snapshot_object(target, filename, savefun=None, \
 *, condition=None, writer=None, snapshot_on_error=False, \
 n_retains=-1, autoload=False)
@@ -159,16 +164,18 @@ n_retains=-1, autoload=False)
                     **kwargs)
 
 
-def snapshot(savefun=None,
-             filename='snapshot_iter_{.iteration}',
-             *,
-             target=None,
-             condition=None,
-             writer=None,
-             snapshot_on_error=False,
-             n_retains=-1,
-             autoload=False,
-             saver_rank=None):
+def snapshot(
+        savefun: Any = None,
+        filename: str = 'snapshot_iter_{.iteration}',
+        *,
+        target: Any = None,
+        condition: Any = None,
+        writer: Optional[writing.Writer] = None,
+        snapshot_on_error: bool = False,
+        n_retains: int = -1,
+        autoload: bool = False,
+        saver_rank: Optional[int] = None,
+) -> '_Snapshot':
     """
     Returns a trainer extension to take snapshots of the trainer.
 
@@ -283,7 +290,7 @@ trigger=(1, 'epoch'))
         autoload=autoload, saver_rank=saver_rank, savefun=savefun)
 
 
-def _always_true():
+def _always_true() -> bool:
     return True
 
 
@@ -306,10 +313,16 @@ class _Snapshot(extension.Extension):
     needs_model_state = True
 
     def __init__(
-            self, target=None, condition=None, writer=None,
-            filename='snapshot_iter_{.iteration}',
-            snapshot_on_error=False, n_retains=-1, autoload=False,
-            savefun=None):
+            self,
+            target: Any = None,
+            condition: Any = None,
+            writer: Optional[writing.Writer] = None,
+            filename: str = 'snapshot_iter_{.iteration}',
+            snapshot_on_error: bool = False,
+            n_retains: int = -1,
+            autoload: bool = False,
+            savefun: Any = None,
+    ) -> None:
         if condition is None:
             condition = _always_true
         self._target = target
@@ -321,7 +334,8 @@ class _Snapshot(extension.Extension):
         self.autoload = autoload
         self._savefun = savefun
 
-    def initialize(self, manager):
+    def initialize(  # type: ignore[override]
+            self, manager: ExtensionsManagerProtocol) -> Optional[str]:
         target = manager if self._target is None else self._target
         outdir = manager.out
         writer = manager.writer if self.writer is None else self.writer
@@ -333,6 +347,7 @@ class _Snapshot(extension.Extension):
             # from ``filename`` format, picks up the latest one in
             # terms of mtime, and tries to load it it the target or
             # manager.
+            assert writer is not None
             loaded_fn = _find_latest_snapshot(self.filename, outdir, writer.fs)
             if loaded_fn:
                 snapshot_file = writer.fs.open(os.path.join(outdir, loaded_fn))
@@ -341,14 +356,13 @@ class _Snapshot(extension.Extension):
                 # ``save_npz`` . In order to support general format,
                 # we nned to first reconstruct the design of savefun
                 # and loadfun.
-                state = torch.load(snapshot_file,
+                state = torch.load(snapshot_file,  # type: ignore[no-untyped-call]
                                    map_location=torch.device("cpu"))
-                kwargs = {}
                 if type(target) is dict:
                     for k in target:
-                        target[k].load_state_dict(state[k], **kwargs)
+                        target[k].load_state_dict(state[k])
                 else:
-                    target.load_state_dict(state, **kwargs)
+                    target.load_state_dict(state)
                 snapshot_file.close()
 
         if (hasattr(writer, '_add_cleanup_hook')
@@ -360,48 +374,54 @@ class _Snapshot(extension.Extension):
             # built-in writer, a cleanup method that is to be
             # triggered right after creation of new snapshot file, is
             # injected here.
-            def _cleanup():
+            def _cleanup() -> None:
+                assert writer is not None
                 files = _find_stale_snapshots(self.filename, outdir,
                                               self.n_retains, writer.fs)
                 for file in files:
                     writer.fs.remove(os.path.join(outdir, file))
 
+            assert writer is not None
             writer._add_cleanup_hook(_cleanup)
 
         return loaded_fn
 
-    def on_error(self, manager, exc, tb):
+    def on_error(
+            self,
+            manager: ExtensionsManagerProtocol,
+            exc: Exception,
+            tb: types.TracebackType,
+    ) -> None:
         super().on_error(manager, exc, tb)
         if self._snapshot_on_error:
             self._make_snapshot(manager)
 
-    def __call__(self, manager):
+    def __call__(self, manager: ExtensionsManagerProtocol) -> None:
         if self.condition():
             self._make_snapshot(manager)
 
-    def _make_snapshot(self, manager):
+    def _make_snapshot(self, manager: ExtensionsManagerProtocol) -> None:
         target = manager if self._target is None else self._target
         writer = manager.writer if self.writer is None else self.writer
         self.writer = writer
         # We need to get a dictionary with the state here
-        kwargs = {}
 
         if type(target) is dict:
             serialized_target = {
-                k: v.state_dict(**kwargs) for k, v in target.items()}
+                k: v.state_dict() for k, v in target.items()}
         else:
-            serialized_target = target.state_dict(**kwargs)
+            serialized_target = target.state_dict()
         filename = self.filename
         if callable(filename):
             filename = filename(manager)
         else:
             filename = filename.format(manager)
         outdir = manager.out
-        writer(filename, outdir, serialized_target, savefun=self._savefun)
+        writer(  # type: ignore
+            filename, outdir, serialized_target, savefun=self._savefun)
 
-    def finalize(self):
-        if hasattr(self.writer, 'finalize'):
-            self.writer.finalize()
+    def finalize(self) -> None:
+        self.writer.finalize()  # type: ignore
 
 
 class _DistributedSnapshot(_Snapshot):
@@ -422,29 +442,36 @@ class _DistributedSnapshot(_Snapshot):
     priority = extension.PRIORITY_SNAPSHOT
 
     def __init__(
-            self, target=None, condition=None, writer=None,
-            filename='snapshot_iter_{.iteration}',
-            snapshot_on_error=False, n_retains=-1, autoload=False,
-            saver_rank=0, savefun=None):
+            self,
+            target: Any = None,
+            condition: Any = None,
+            writer: Optional[writing.Writer] = None,
+            filename: str = 'snapshot_iter_{.iteration}',
+            snapshot_on_error: bool = False,
+            n_retains: int = -1,
+            autoload: bool = False,
+            saver_rank: int = 0,
+            savefun: Any = None,
+    ):
         super().__init__(target, condition, writer, filename,
                          snapshot_on_error, n_retains,
                          autoload, savefun)
         # To support distributed snapshots
-        if not torch.distributed.is_initialized():
+        if not torch.distributed.is_initialized():  # type: ignore[no-untyped-call]
             raise RuntimeError('The Distributed Snapshot extension',
                                ' requires torch.distributed to be initialized')
         self._saver_rank = saver_rank
-        self._size = torch.distributed.get_world_size()
-        self._rank = torch.distributed.get_rank()
+        self._size = torch.distributed.get_world_size()  # type: ignore[no-untyped-call]
+        self._rank = torch.distributed.get_rank()  # type: ignore[no-untyped-call]
         if not (0 <= saver_rank < self._size):
             raise ValueError('Distributed snapshot requires a saver rank'
                              ' in the range [0-{})'.format(self._size))
 
-    def __call__(self, trainer):
+    def __call__(self, manager: ExtensionsManagerProtocol) -> None:
         if self.condition():
             # on distributed environments only the designed rank
             # saves the snapshot
             if self._rank == self._saver_rank:
-                self._make_snapshot(trainer)
+                self._make_snapshot(manager)
             if self._size > 1:
-                torch.distributed.barrier()
+                torch.distributed.barrier()  # type: ignore[no-untyped-call]
