@@ -10,6 +10,7 @@ import warnings
 
 import torch
 
+import pytorch_pfn_extras
 from pytorch_pfn_extras import writing
 from pytorch_pfn_extras import reporting
 from pytorch_pfn_extras.training import extension as extension_module
@@ -31,18 +32,13 @@ class IterationNotification:
         self._is_completed = False
 
 
-class _ManagerExecutionProxy:
-    """
-    Object that is passed to the triggers of extensions depending if they
-    require to measure executions when using async mode
-    Note that in sync mode execution == iteration
-    """
+class _ManagerProxy:
     def __init__(self, manager: '_BaseExtensionsManager') -> None:
         self._manager = manager
 
     @property
     def iteration(self) -> int:
-        return self._manager.execution
+        return self._manager.iteration
 
     @property
     def epoch(self) -> int:
@@ -60,15 +56,15 @@ class _ManagerExecutionProxy:
 
     @property
     def models(self) -> Dict[str, torch.nn.Module]:
-        raise RuntimeError('Models are not available during execution phase.')
+        return self._manager.models
 
     @property
     def raw_models(self) -> Mapping[str, torch.nn.Module]:
-        raise RuntimeError('Models are not available during execution phase.')
+        return self._manager.raw_models
 
     @property
     def optimizers(self) -> Mapping[str, torch.optim.Optimizer]:
-        raise RuntimeError('Optimizers are not available during execution phase.')
+        return self._manager.optimizers
 
     @property
     def elapsed_time(self) -> float:
@@ -82,6 +78,10 @@ class _ManagerExecutionProxy:
     def stop_trigger(self) -> bool:
         return self._manager.stop_trigger
 
+    @property
+    def _stop_trigger(self) -> trigger_module.Trigger:
+        return self._manager._stop_trigger
+
     @property
     def out(self) -> str:
         return self._manager.out
@@ -97,6 +97,33 @@ class _ManagerExecutionProxy:
     def get_extension(self, name: str) -> extension_module.Extension:
         return self._manager.get_extension(name)
 
+    @property
+    def observation(self) -> reporting.Observation:
+        return self._manager.observation
+
+
+class _ManagerExecutionProxy(_ManagerProxy):
+    """
+    Object that is passed to the triggers of extensions depending if they
+    require to measure executions when using async mode
+    Note that in sync mode execution == iteration
+    """
+    @property
+    def iteration(self) -> int:
+        return self._manager.execution
+
+    @property
+    def models(self) -> Dict[str, torch.nn.Module]:
+        raise RuntimeError('Models are not available during execution phase.')
+
+    @property
+    def raw_models(self) -> Mapping[str, torch.nn.Module]:
+        raise RuntimeError('Models are not available during execution phase.')
+
+    @property
+    def optimizers(self) -> Mapping[str, torch.optim.Optimizer]:
+        raise RuntimeError('Optimizers are not available during execution phase.')
+
 
 class _BaseExtensionsManager:
     """
@@ -501,12 +528,30 @@ class _BaseExtensionsManager:
                                  for name in self._optimizers}
         to_save['extensions'] = {name: self._extensions[name].state_dict()
                                  for name in self._extensions}
+        to_save['ppe_version'] = pytorch_pfn_extras.__version__
         return to_save
 
+    def _check_snapshot_version(self, ppe_version: Optional[str]) -> None:
+        must_warn = ppe_version is None or (
+            ppe_version != pytorch_pfn_extras.__version__)
+
+        if not must_warn:
+            return
+
+        msg = ('You are trying to load a snapshot file taken using a different '
+               'PPE version.\n')
+
+        if ppe_version is not None:
+            msg += (f'Snapshot taken with PPE {ppe_version} but '
+                    f'currently using PPE {pytorch_pfn_extras.__version__}')
+
+        warnings.warn(msg)
+
     def load_state_dict(
             self,
             to_load: Dict[str, Any],
     ) -> None:
+        self._check_snapshot_version(to_load.get('ppe_version', None))
         self._start_iteration = to_load['_start_iteration']
         self.iteration = self._start_iteration
         self._start_execution = to_load.get('_start_execution', self.iteration)
@@ -649,6 +694,7 @@ class ExtensionsManager(_BaseExtensionsManager):
 
         if self._internal_stop_trigger(self):
             self._finalize_extensions()
+            self.writer.finalize()
 
 
 if TYPE_CHECKING:
